@@ -1,6 +1,6 @@
 // This module implements the portability layer for the Qt port of Scintilla.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -51,21 +51,23 @@
 #include "Qsci/qsciscintillabase.h"
 #include "SciClasses.h"
 
+#include "FontQuality.h"
+
 
 // Type convertors.
-static QFont *PFont(FontID id)
+static QFont *PFont(FontID fid)
 {
-    return reinterpret_cast<QFont *>(id);
+    return reinterpret_cast<QFont *>(fid);
 }
 
-static QWidget *PWindow(WindowID id)
+static QWidget *PWindow(WindowID wid)
 {
-    return reinterpret_cast<QWidget *>(id);
+    return reinterpret_cast<QWidget *>(wid);
 }
 
-static SciPopup *PMenu(MenuID id)
+static SciPopup *PMenu(MenuID mid)
 {
-    return reinterpret_cast<SciPopup *>(id);
+    return reinterpret_cast<SciPopup *>(mid);
 }
 
 
@@ -107,7 +109,7 @@ void Palette::Allocate(Window &)
 
 
 // Font management.
-Font::Font() : id(0)
+Font::Font() : fid(0)
 {
 }
 
@@ -116,15 +118,43 @@ Font::~Font()
 }
 
 void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
-        bool)
+        int flags)
 {
     Release();
 
     QFont *f = new QFont();
 
+    QFont::StyleStrategy strategy;
+
+    switch (flags & SC_EFF_QUALITY_MASK)
+    {
+    case SC_EFF_QUALITY_NON_ANTIALIASED:
+        strategy = QFont::NoAntialias;
+        break;
+
+    case SC_EFF_QUALITY_ANTIALIASED:
+        strategy = QFont::PreferAntialias;
+        break;
+
+    default:
+        strategy = QFont::PreferDefault;
+    }
+
+#if defined(Q_WS_MAC)
+#if QT_VERSION >= 0x040700
+    strategy = static_cast<QFont::StyleStrategy>(strategy | QFont::ForceIntegerMetrics);
+#else
+#warning "Correct handling of QFont metrics requires Qt v4.7.0 or later"
+#endif
+#endif
+
+    f->setStyleStrategy(strategy);
+
     // If name of the font begins with a '-', assume, that it is an XLFD.
     if (faceName[0] == '-')
+    {
         f->setRawName(faceName);
+    }
     else
     {
         f->setFamily(faceName);
@@ -133,15 +163,15 @@ void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
         f->setItalic(italic);
     }
 
-    id = f;
+    fid = f;
 }
 
 void Font::Release()
 {
-    if (id)
+    if (fid)
     {
-        delete PFont(id);
-        id = 0;
+        delete PFont(fid);
+        fid = 0;
     }
 }
 
@@ -648,7 +678,7 @@ Window::~Window()
 
 void Window::Destroy()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
     {
@@ -659,18 +689,18 @@ void Window::Destroy()
         // around the problem but this is the simplest and doesn't seem to
         // cause problems of its own.
         w->deleteLater();
-        id = 0;
+        wid = 0;
     }
 }
 
 bool Window::HasFocus()
 {
-    return PWindow(id)->hasFocus();
+    return PWindow(wid)->hasFocus();
 }
 
 PRectangle Window::GetPosition()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     // Before any size allocated pretend its big enough not to be scrolled.
     PRectangle rc(0,0,5000,5000);
@@ -688,19 +718,19 @@ PRectangle Window::GetPosition()
 
 void Window::SetPosition(PRectangle rc)
 {
-    PWindow(id)->setGeometry(rc.left, rc.top, rc.right - rc.left,
+    PWindow(wid)->setGeometry(rc.left, rc.top, rc.right - rc.left,
             rc.bottom - rc.top);
 }
 
 void Window::SetPositionRelative(PRectangle rc, Window relativeTo)
 {
-    QWidget *rel = PWindow(relativeTo.id);
+    QWidget *rel = PWindow(relativeTo.wid);
     QPoint pos = rel->mapToGlobal(rel->pos());
 
     int x = pos.x() + rc.left;
     int y = pos.y() + rc.top;
 
-    PWindow(id)->setGeometry(x, y, rc.right - rc.left, rc.bottom - rc.top);
+    PWindow(wid)->setGeometry(x, y, rc.right - rc.left, rc.bottom - rc.top);
 }
 
 PRectangle Window::GetClientPosition()
@@ -710,7 +740,7 @@ PRectangle Window::GetClientPosition()
 
 void Window::Show(bool show)
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (show)
         w->show();
@@ -720,7 +750,7 @@ void Window::Show(bool show)
 
 void Window::InvalidateAll()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
         w->update();
@@ -728,7 +758,7 @@ void Window::InvalidateAll()
 
 void Window::InvalidateRectangle(PRectangle rc)
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
         w->update(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
@@ -736,7 +766,7 @@ void Window::InvalidateRectangle(PRectangle rc)
 
 void Window::SetFont(Font &font)
 {
-    PWindow(id)->setFont(*PFont(font.GetID()));
+    PWindow(wid)->setFont(*PFont(font.GetID()));
 }
 
 void Window::SetCursor(Cursor curs)
@@ -770,15 +800,17 @@ void Window::SetCursor(Cursor curs)
         break;
 
     default:
+        // Note that Qt doesn't have a standard cursor that could be used to
+        // implement cursorReverseArrow.
         qc = Qt::ArrowCursor;
     }
 
-    PWindow(id)->setCursor(qc);
+    PWindow(wid)->setCursor(qc);
 }
 
 void Window::SetTitle(const char *s)
 {
-    PWindow(id)->setCaption(s);
+    PWindow(wid)->setCaption(s);
 }
 
 
@@ -789,30 +821,30 @@ PRectangle Window::GetMonitorRect(Point)
 
 
 // Menu management.
-Menu::Menu() : id(0)
+Menu::Menu() : mid(0)
 {
 }
 
 void Menu::CreatePopUp()
 {
     Destroy();
-    id = new SciPopup();
+    mid = new SciPopup();
 }
 
 void Menu::Destroy()
 {
-    SciPopup *m = PMenu(id);
+    SciPopup *m = PMenu(mid);
 
     if (m)
     {
         delete m;
-        id = 0;
+        mid = 0;
     }
 }
 
 void Menu::Show(Point pt, Window &)
 {
-    PMenu(id)->popup(QPoint(pt.x, pt.y));
+    PMenu(mid)->popup(QPoint(pt.x, pt.y));
 }
 
 
